# Copyright (C) 2018 Matthew L. Schwartz - all rights reserved
# Not for redistribution

from tkinter import messagebox

# Text in dialog called in SapTrap Builder file menu -> About SapTrap Builder
aboutSapTrap = {'Title': 'SapTrap Builder',
                'Text':
                'Version : 5.0\nCopyright (C) 2018 Matthew L. Schwartz'
                '\n\nPython 3.6 - Copyright (C) 2001-2016 Python Software Foundation; All Rights Reserved'
                '\nTcl/Tk Copyright (C) This software is copyrighted by the Regents of the '
                '\nUniversity of California, Sun Microsystems, Inc., Scriptics Corporation, and other parties. '}

# Text along bottom panel that displays instructions for current step
program_instructions_labels = \
             {'Sequence Entry':
              '1. Enter DNA sequence (CAPS for coding, lowercase for non-coding sequence).'
              '\tcommand+s = strip non-DNA characters'
              '\n2. (Optional) Mark insert site with / , or a region to delete with <....>.'
              '\t\tcommand+u = selection to UPPER CASE'
              '\n3. Press Tab, \'>\', or Step button to advance'
              '\t\t\t\t\tcommand+l = selection to lower case',

              'Select Translation Frame':
              'Left, Right arrow keys = shift translation frame +- 1.',

              'Select Tag Type':
              'Select tag type from dropdown list.\nUse Up and Down arrow keys to scroll through options.',

              'Set Insert Site':
              'Left, Right arrow keys = move insert site +- 1 base. (CAPS = in frame)'
              '\nShift+Left, Shift+Right arrow keys = move insert site +- 9 bases.'
              '\nr = reset to original position \tc = set insert site to cut site of selected gRNA',

              'Select gRNA':
              'Left, Right arrow keys = select next nearest gRNA candidate.',

              'Mutate Repair Template':
              'Up, Down = synonymous codons\t\t\ta, c, g, t = change base\tr = session reset'
              '\nShift+Up, Shift+Down = non-synonymous codons\t- = delete base\t\tz = reset to wild type '
              '\nLeft, Right = Navigate (+Shift for 9 base steps)\tA, C, D, ... = change amino acid\t'
              '\nCommand+(Up, Down, Left, Right) = Find silent restriction sites',

              'Find Silent Sites':
              'Entered Find Silent Sites mode.',

              'Review Design':
              'Repair Template trimmed to specified length.',

              'Get Reagent Sequences':
              ''
              }

flags = {'SapI in guide': 'gRNA sequence contains SapI site. Sequence incompatible with SapTrap assembly.',
         'Terminator in guide': 'gRNA sequence contains a pol III terminator. Incompatible with in vivo production.',
         'Cas9 ends in C': 'gRNA binding site ends in C base. Such guides exhibit low activity in C. elegans.',
         'intact gRNA site': 'gRNA binding site intact. Repair template can be cut by Cas9 + guideRNA.',
         'weak gRNA disruption': 'gRNA binding weakly disrupted. Consider mutations nearer to PAM.',
         'short hUS': 'hUS shorter that desired: insufficient sequence provided for full length hUS.',
         'short hDS': 'hUS shorter that desired: insufficient sequence provided for full length hUS.',
         'long hUS': 'hUS extended beyond target length; cutsite within minimum distance of edge.',
         'long hDS': 'hDS extended beyond target length; cutsite within minimum distance of edge.',
         'Trap enzyme in repair template': 'SapI site in repair template. Mutate SapI site for SapTrap assembly.',
         'Unlikely point mutants': 'Mutations marked with x are less likely to be incorporated in genome.'
         ' Place critical mutations between the insert site and the cutsite.'}


class Error_Handling:
    def __init__(self, master, main_window):
        self.master = master
        self.main_window = main_window


    def error_message(self, error_type):
        """ Input is a string keyword for the type of error. Using this value, responses selected from a list.
            Generally, displays messagebox describing error and how to fix. """
        title = 'Error'
        text = 'Error code not specified.'

        if error_type == 'test_error':
            title = 'Test Error'
            text = 'This is a test error to work out error handling protocols.'

        elif error_type == 'Non-DNA Input':
            title = 'Input Error'
            text = 'Entry sequence can only contain only:\n1. DNA base characters ' \
                                            '\n2. New line characters (returns)\n3. A single, ' \
                                            'optional / to mark the target insertion site.' \
                                            '\n4. The characters < and > to mark the start and end of a ' \
                                            'large intended deletion.'

        elif error_type == 'No DNA Input':
            title = 'Input Error'
            text = 'Please enter a DNA sequence for the target site.'

        elif error_type == 'No Tag Type':
            title = 'Tag type'
            text = 'Please select a tag type (step 3)'

        elif error_type == 'No guide RNA':
            title = 'guide RNA'
            text = 'Please select a guide RNA (step 5)'

        elif error_type == 'Non-integer input':
            title = 'Non-integer input'
            text = 'hUS Target Length, hDS Target Length and min True Length fields must be integers.'

        elif error_type == 'No guides available':
            title = 'No Guides Available'
            text = 'This sequence does not contain any gRNA binding sites!'

        return messagebox.showerror(title, text)
